import type { ZodSchema } from "zod";
import { type RelayActionIdentifier, type RelayActionInput, type RelayActionOutput, type RelayEventIdentifier, type RelayEventPayload, type RelayViewIdentifier, type RelayViewParams, type RelayViewResponse } from "../index";
/**
 * Custom error class for RelayClient errors.
 */
declare class RelayClientError extends Error {
    details?: string;
    original?: Error;
    constructor(message: string, details: string | Error);
}
/**
 * Configuration options for the RelayClient.
 */
interface RelayClientOptions {
    socketHost?: string;
    socketPort?: number;
    socketProtocol?: string;
    socketPath?: string;
    socketQuery?: string;
    httpHost?: string;
    httpPort?: number;
    httpProtocol?: string;
    httpPath?: string;
    httpQuery?: string;
    debugMode?: boolean;
    onError?: ((error: RelayClientError) => void) | null;
}
/**
 * Type definition for event handlers based on the event identifier.
 */
type EventHandlerFor<K extends RelayEventIdentifier> = (payload: RelayEventPayload<K>) => void;
/**
 * The `RelayClient` class provides a client interface for interacting with the Stardust Relay.
 *
 * @example
 * ```typescript
 * const client = new RelayClient('your-token', {
 *   debugMode: true,
 *   onError: (error) => console.error(error),
 * });
 *
 * client.on('some:event', (eventData) => {
 *   console.log('Received someEvent:', eventData);
 * });
 *
 * await client.view('some/view', { param1: 'value1' });
 * await client.action('some@action', { input1: 'value1' });
 * ```
 *
 * @class
 * @param {string} token - The authentication token for the Relay service.
 * @param {RelayClientOptions} [options] - Optional configuration options for the client.
 * @param {string} [options.socketHost="relay-ws.astral.camp"] - The WebSocket host.
 * @param {number} [options.socketPort=443] - The WebSocket port.
 * @param {string} [options.socketProtocol="wss"] - The WebSocket protocol.
 * @param {string} [options.socketPath="/ws"] - The WebSocket path.
 * @param {string} [options.socketQuery=""] - The WebSocket query string.
 * @param {string} [options.httpHost="relay.astral.camp"] - The HTTP host.
 * @param {number} [options.httpPort=443] - The HTTP port.
 * @param {string} [options.httpProtocol="https"] - The HTTP protocol.
 * @param {boolean} [options.debugMode=false] - Whether to enable debug mode.
 * @param {(error: RelayClientError) => void} [options.onError=null] - Optional error handler.
 */
export declare class RelayClient {
    private socket;
    private eventHandlers;
    private errorHandler;
    private apiURL;
    private token;
    private debugMode;
    isConnected: boolean;
    constructor(token: string, { socketHost, socketPort, socketProtocol, socketPath, socketQuery, httpHost, httpPort, httpProtocol, debugMode, onError, }?: RelayClientOptions);
    /**
     * Logs debug messages to the console if debug mode is enabled.
     *
     * @param {...unknown[]} message - The messages to log.
     */
    private debugMessage;
    /**
     * Sends a POST request to the specified path with the provided request body.
     *
     * @template RequestBody - The type of the request body.
     * @template ResponseBody - The type of the response body.
     *
     * @param {string} path - The path to which the POST request is sent.
     * @param {RequestBody} body - The request body to be sent.
     * @param {ZodSchema<ResponseBody>} [bodySchema] - Optional schema to validate the response body.
     *
     * @returns {Promise<ResponseBody>} - A promise that resolves to the response body.
     *
     * @throws {RelayClientError} - Throws an error if the request fails or if the response body fails schema validation.
     */
    protected post<RequestBody, ResponseBody>(path: string, body: RequestBody, bodySchema?: ZodSchema<ResponseBody>): Promise<ResponseBody>;
    /**
     * Handles incoming messages by parsing them as Relay events and processing them accordingly.
     *
     * @param message - The incoming message to be handled. It is expected to be of an unknown type.
     *
     * @throws {RelayClientError} Throws an error if the message is invalid and cannot be parsed as a Relay event.
     */
    protected handleMessage(message: unknown): void;
    /**
     * Handles a relay event by invoking all registered event handlers for the specified event.
     *
     * @template Event - The type of the relay event name.
     * @param eventName - The name of the event to handle.
     * @param eventData - The data associated with the event.
     */
    protected handleEvent<Event extends RelayEventIdentifier>(eventName: Event, eventData: RelayEventPayload<Event>): void;
    /**
     * Registers an event handler for a specified event.
     *
     * @template Event - The type of the event name.
     * @param {Event} event - The name of the event to listen for.
     * @param {EventHandlerFor<Event>} handler - The handler function to be called when the event is emitted.
     */
    addEventListener<Event extends RelayEventIdentifier>(event: Event, handler: EventHandlerFor<Event>): () => void;
    /**
     * Removes an event listener for the specified event.
     *
     * @template Event - The type of the event name.
     * @param {Event} event - The name of the event to remove the listener from.
     * @param {EventPayloadHandlersByName[Event]} handler - The handler function to remove.
     * @returns {void}
     */
    removeEventListener<Event extends RelayEventIdentifier>(event: Event, handler: EventHandlerFor<Event>): void;
    /**
     * Fetches and returns the response for a specified view.
     *
     * @template View - The name of the view to fetch.
     * @param view - The name of the view to fetch.
     * @param params - The parameters required for the view.
     * @returns A promise that resolves to the response of the specified view.
     * @throws {RelayClientError} If the schema for the specified view is not found.
     */
    view<View extends RelayViewIdentifier>(view: View, params: RelayViewParams<View>): Promise<RelayViewResponse<View>>;
    /**
     * Executes a specified relay action with the given input and returns the output.
     *
     * @template Action - The type of the relay action name.
     * @param {Action} action - The name of the relay action to execute.
     * @param {RelayActionInput<Action>} input - The input data for the relay action.
     * @returns {Promise<RelayActionOutput<Action>>} - A promise that resolves to the output of the relay action.
     * @throws {RelayClientError} - Throws an error if the action is invalid, the action fails, or the output fails schema validation.
     */
    action<Action extends RelayActionIdentifier>(action: Action, input: RelayActionInput<Action>): Promise<RelayActionOutput<Action>>;
}
export {};
