import { ManagedSocket } from "@endeavorance/socket";
import { GenericRelayEventSchema, RelayActionResultSchema, RelayActions, RelayEvents, RelayViews, } from "../index";
/**
 * Custom error class for RelayClient errors.
 */
class RelayClientError extends Error {
    constructor(message, details) {
        super(message);
        this.name = "RelayClientError";
        this.details = typeof details === "string" ? details : details?.message;
        this.original = details instanceof Error ? details : new Error(details);
    }
}
function isValidEventIdentifier(id) {
    return Object.keys(RelayEvents).includes(id);
}
function isValidViewIdentifier(id) {
    return Object.keys(RelayViews).includes(id);
}
// Creates a map of event names to empty arrays of event handlers
function createEmtpyEventHandlerMap() {
    const obj = {};
    const events = Object.keys(RelayEvents);
    for (const option of events) {
        obj[option] = [];
    }
    return obj;
}
/**
 * The `RelayClient` class provides a client interface for interacting with the Stardust Relay.
 *
 * @example
 * ```typescript
 * const client = new RelayClient('your-token', {
 *   debugMode: true,
 *   onError: (error) => console.error(error),
 * });
 *
 * client.on('some:event', (eventData) => {
 *   console.log('Received someEvent:', eventData);
 * });
 *
 * await client.view('some/view', { param1: 'value1' });
 * await client.action('some@action', { input1: 'value1' });
 * ```
 *
 * @class
 * @param {string} token - The authentication token for the Relay service.
 * @param {RelayClientOptions} [options] - Optional configuration options for the client.
 * @param {string} [options.socketHost="relay-ws.astral.camp"] - The WebSocket host.
 * @param {number} [options.socketPort=443] - The WebSocket port.
 * @param {string} [options.socketProtocol="wss"] - The WebSocket protocol.
 * @param {string} [options.socketPath="/ws"] - The WebSocket path.
 * @param {string} [options.socketQuery=""] - The WebSocket query string.
 * @param {string} [options.httpHost="relay.astral.camp"] - The HTTP host.
 * @param {number} [options.httpPort=443] - The HTTP port.
 * @param {string} [options.httpProtocol="https"] - The HTTP protocol.
 * @param {boolean} [options.debugMode=false] - Whether to enable debug mode.
 * @param {(error: RelayClientError) => void} [options.onError=null] - Optional error handler.
 */
export class RelayClient {
    constructor(token, { socketHost = "relay-ws.astral.camp", socketPort = 443, socketProtocol = "wss", socketPath = "/ws", socketQuery = "", httpHost = "relay.astral.camp", httpPort = 443, httpProtocol = "https", debugMode = false, onError = null, } = {}) {
        this.eventHandlers = createEmtpyEventHandlerMap();
        this.errorHandler = null;
        this.debugMode = false;
        this.isConnected = false;
        this.token = token;
        this.debugMode = debugMode;
        this.errorHandler = onError;
        // Url to connect to the Relay socket server (events)
        const socketConnURL = new URL(`${socketProtocol}://${socketHost}:${socketPort}${socketPath}`);
        socketConnURL.search = socketQuery;
        socketConnURL.searchParams.set("token", token);
        // Url to use for fetch requests to the Relay API server (actions/views)
        this.apiURL = new URL(`${httpProtocol}://${httpHost}:${httpPort}`).toString();
        // Set up the socket connection
        this.socket = new ManagedSocket(socketConnURL.toString());
        this.socket.debug = debugMode;
        this.socket.onMessage(this.handleMessage.bind(this));
        this.socket.onError((error) => {
            const wrappedError = new RelayClientError("Socket error", error);
            if (this.errorHandler) {
                return this.errorHandler(wrappedError);
            }
            this.debugMessage("[Client]", "Socket error", error);
            throw wrappedError;
        });
        this.socket.onOpen(() => {
            this.isConnected = true;
            this.debugMessage("Relay Socket connected");
        });
        this.socket.onClose(() => {
            this.isConnected = false;
            this.debugMessage("Relay Socket disconnected");
        });
        this.socket.onCatastrophicError((error) => {
            const wrappedError = new RelayClientError("Socket catastrophic error", error);
            if (this.errorHandler) {
                return this.errorHandler(wrappedError);
            }
            this.debugMessage("[Client]", "Socket catastrophic error", error);
            throw wrappedError;
        });
    }
    /**
     * Logs debug messages to the console if debug mode is enabled.
     *
     * @param {...unknown[]} message - The messages to log.
     */
    debugMessage(...message) {
        if (this.debugMode) {
            console.log("[Client]", ...message);
        }
    }
    /**
     * Sends a POST request to the specified path with the provided request body.
     *
     * @template RequestBody - The type of the request body.
     * @template ResponseBody - The type of the response body.
     *
     * @param {string} path - The path to which the POST request is sent.
     * @param {RequestBody} body - The request body to be sent.
     * @param {ZodSchema<ResponseBody>} [bodySchema] - Optional schema to validate the response body.
     *
     * @returns {Promise<ResponseBody>} - A promise that resolves to the response body.
     *
     * @throws {RelayClientError} - Throws an error if the request fails or if the response body fails schema validation.
     */
    async post(path, body, bodySchema) {
        const url = new URL(path, this.apiURL).toString();
        const response = await fetch(url, {
            method: "POST",
            headers: {
                "Content-Type": "application/json",
                Authorization: `Bearer ${this.token}`,
            },
            body: JSON.stringify(body),
        });
        if (!response.ok) {
            const responseText = await response.text();
            this.debugMessage(responseText);
            throw new RelayClientError(`Failed to fetch ${url}: ${response.status} - ${response.statusText}`, responseText);
        }
        // If a body schema is provided, parse the response body for added safety
        if (bodySchema) {
            const responseBody = await response.json();
            const parsedBody = bodySchema.safeParse(responseBody);
            if (!parsedBody.success) {
                throw new RelayClientError(`API response failed schema check: ${url}\n\Validation output: ${JSON.stringify(parsedBody.error, null, 2)}\n\nRaw Response: ${JSON.stringify(responseBody, null, 2)}`, JSON.stringify(responseBody));
            }
            return parsedBody.data;
        }
        return (await response.json());
    }
    /**
     * Handles incoming messages by parsing them as Relay events and processing them accordingly.
     *
     * @param message - The incoming message to be handled. It is expected to be of an unknown type.
     *
     * @throws {RelayClientError} Throws an error if the message is invalid and cannot be parsed as a Relay event.
     */
    handleMessage(message) {
        const parsedAsEvent = GenericRelayEventSchema.safeParse(message);
        if (!parsedAsEvent.success) {
            throw new RelayClientError("Received invalid message", JSON.stringify(message));
        }
        const { eventName, payload } = parsedAsEvent.data;
        if (this.debugMode) {
            console.log("[Client]", "Received message", {
                eventName,
                payload,
            });
        }
        if (!isValidEventIdentifier(eventName)) {
            throw new RelayClientError("Invalid event name", eventName);
        }
        const payloadSchema = RelayEvents[eventName];
        const parsedPayload = payloadSchema.safeParse(payload);
        if (!parsedPayload.success) {
            throw new RelayClientError("Invalid event payload", JSON.stringify(payload));
        }
        this.handleEvent(eventName, payload);
        return;
    }
    /**
     * Handles a relay event by invoking all registered event handlers for the specified event.
     *
     * @template Event - The type of the relay event name.
     * @param eventName - The name of the event to handle.
     * @param eventData - The data associated with the event.
     */
    handleEvent(eventName, eventData) {
        const handlersForEvent = this.eventHandlers[eventName] ?? [];
        for (const handler of handlersForEvent) {
            handler(eventData);
        }
    }
    /**
     * Registers an event handler for a specified event.
     *
     * @template Event - The type of the event name.
     * @param {Event} event - The name of the event to listen for.
     * @param {EventHandlerFor<Event>} handler - The handler function to be called when the event is emitted.
     */
    addEventListener(event, handler) {
        if (!this.eventHandlers[event]) {
            this.eventHandlers[event] = [];
        }
        this.eventHandlers[event].push(handler);
        return () => {
            this.removeEventListener(event, handler);
        };
    }
    /**
     * Removes an event listener for the specified event.
     *
     * @template Event - The type of the event name.
     * @param {Event} event - The name of the event to remove the listener from.
     * @param {EventPayloadHandlersByName[Event]} handler - The handler function to remove.
     * @returns {void}
     */
    removeEventListener(event, handler) {
        if (!this.eventHandlers[event]) {
            return;
        }
        const index = this.eventHandlers[event].indexOf(handler);
        if (index !== -1) {
            this.eventHandlers[event].splice(index, 1);
        }
    }
    /**
     * Fetches and returns the response for a specified view.
     *
     * @template View - The name of the view to fetch.
     * @param view - The name of the view to fetch.
     * @param params - The parameters required for the view.
     * @returns A promise that resolves to the response of the specified view.
     * @throws {RelayClientError} If the schema for the specified view is not found.
     */
    async view(view, params) {
        if (!isValidViewIdentifier(view)) {
            throw new RelayClientError(`Invalid view: ${view}`, "Failed to find schema for view");
        }
        const castSchema = RelayViews[view].response;
        return this.post(`/view/${view}`, params, castSchema);
    }
    /**
     * Executes a specified relay action with the given input and returns the output.
     *
     * @template Action - The type of the relay action name.
     * @param {Action} action - The name of the relay action to execute.
     * @param {RelayActionInput<Action>} input - The input data for the relay action.
     * @returns {Promise<RelayActionOutput<Action>>} - A promise that resolves to the output of the relay action.
     * @throws {RelayClientError} - Throws an error if the action is invalid, the action fails, or the output fails schema validation.
     */
    async action(action, input) {
        const schema = RelayActions[action];
        if (!schema) {
            throw new RelayClientError(`Invalid Action: ${action}`, "Failed to find schema for view");
        }
        const castSchema = schema.output;
        const actionResult = await this.post(`/action/${action}`, input, RelayActionResultSchema);
        if (!actionResult.success) {
            throw new RelayClientError("Action failed", actionResult.errorId);
        }
        const { payload } = actionResult;
        const parsed = castSchema.safeParse(payload);
        if (!parsed.success) {
            throw new RelayClientError(`Action output failed schema check: ${action}\n\Validation output: ${JSON.stringify(parsed.error, null, 2)}\n\nRaw Response: ${JSON.stringify(payload, null, 2)}`, JSON.stringify(payload));
        }
        return parsed.data;
    }
}
