import { z } from "zod";
import { pickOne } from "../util";
import { FlowerImages } from "./assets";
import { getItemDefinition, } from "./items";
export const MAX_FLOWERS_PER_USER = 10;
export const MAX_FLOWER_WATER_LEVEL = 100;
export const FLOWER_SLUGS = [
    "astralDaisy",
    "voidIvy",
    "crystalOrchid",
    "bloodLily",
    "obsidianRose",
    "rainbowLotus",
    "dragonleaf",
    "wanderingIce",
    "lightningLotus",
    "smoulderingRose",
    "stardustBlossom",
    "celestialBulb",
    "celestialBoonflower",
    "astralMangoTree",
    "spectralPapayaTree",
    "ritualPumpkin",
    "acidBlossom",
    "trophyThistle",
    "joyroot",
    "joyblossom",
    "joybrushTangle",
];
export const FlowerSlugSchema = z.enum(FLOWER_SLUGS);
export const BotanyPlantSchema = z.object({
    slug: z.enum(FLOWER_SLUGS),
    id: z.number(),
    ownerId: z.number(),
    plantedAt: z.number(),
    waterLevel: z.number(),
    waterMissing: z.number(),
    waterRate: z.number(),
    ticksRemaining: z.number(),
    name: z.string(),
    description: z.string(),
    icon: z.string(),
    isGrowing: z.boolean(),
    isWaterable: z.boolean(),
    isDead: z.boolean(),
    isReady: z.boolean(),
});
export const BotanyCommunityGardenSchema = z.record(z.number(), z.array(BotanyPlantSchema));
/**
 * Given a seed item, pick a flower to plant
 * @param slug The seed item to pick a flower from
 * @returns The slug of the flower to plant
 */
export function getFlowerFromSeed(slug) {
    const seedItemDefinition = getItemDefinition(slug);
    if (!seedItemDefinition.relatedFlower) {
        throw new Error("Cannot get a flower from this item type");
    }
    const flowerOptions = Array.isArray(seedItemDefinition.relatedFlower)
        ? seedItemDefinition.relatedFlower
        : [seedItemDefinition.relatedFlower];
    return pickOne(flowerOptions);
}
const FLOWER_DEFINITIONS = {
    astralDaisy: {
        name: "Astral Daisy",
        slug: "astralDaisy",
        description: "A simple little specral flower.",
        destroyOnHarvest: true,
        waterConsumptionRate: 4,
        growingTicks: 5,
        icon: FlowerImages.AstralDaisy,
        loot: {
            conduitDropMin: 5,
            conduitDropMax: 25,
            possibleItemDrops: [
                "mundaneSeed",
                "crystalGeode",
                "treasureChest",
                "quantumSeed",
                "stardustBag",
            ],
            guaranteedItemDrops: ["harvestedAstralDaisy"],
            itemDropMin: 0,
            itemDropMax: 1,
        },
    },
    voidIvy: {
        name: "Void Ivy",
        slug: "voidIvy",
        description: "An inky-black web of ivy. It drips with some kind of dark resin.",
        destroyOnHarvest: true,
        waterConsumptionRate: 7,
        growingTicks: 14,
        icon: FlowerImages.VoidIvy,
        loot: {
            conduitDropMin: 25,
            conduitDropMax: 60,
            possibleItemDrops: ["voidIvySeed", "stardustBag"],
            guaranteedItemDrops: ["harvestedVoidIvy"],
            itemDropMin: 0,
            itemDropMax: 1,
        },
    },
    crystalOrchid: {
        name: "Crystal Orchid",
        slug: "crystalOrchid",
        description: "A beautiful but delicate crystal flower.",
        destroyOnHarvest: true,
        waterConsumptionRate: 2,
        growingTicks: 100,
        icon: FlowerImages.CrystalOrchid,
        loot: {
            conduitDropMin: 700,
            conduitDropMax: 800,
            possibleItemDrops: ["crystalGeode", "crystalOrchidSeed"],
            guaranteedItemDrops: ["crystalGeode", "harvestedCrystalOrchid"],
            itemDropMin: 1,
            itemDropMax: 3,
        },
    },
    bloodLily: {
        name: "Blood Lily",
        slug: "bloodLily",
        description: "A rapidly growing and demanding flower. It almost seems sentient.",
        destroyOnHarvest: true,
        waterConsumptionRate: 95,
        growingTicks: 2,
        icon: FlowerImages.BloodLily,
        loot: {
            conduitDropMin: 50,
            conduitDropMax: 100,
            possibleItemDrops: ["bloodLilySeed", "stardustBag"],
            guaranteedItemDrops: ["harvestedBloodLily"],
            itemDropMin: 1,
            itemDropMax: 2,
        },
    },
    obsidianRose: {
        name: "Obsidian Rose",
        slug: "obsidianRose",
        description: "Slow growing, but hearty.",
        destroyOnHarvest: true,
        waterConsumptionRate: 3,
        growingTicks: 168,
        icon: FlowerImages.ObsidianRose,
        loot: {
            conduitDropMin: 300,
            conduitDropMax: 450,
            possibleItemDrops: [
                "obsidianRoseSeed",
                "fallenStar",
                "cracklingRainmeter",
            ],
            guaranteedItemDrops: ["harvestedObsidianRose"],
            itemDropMin: 1,
            itemDropMax: 2,
        },
    },
    smoulderingRose: {
        name: "Smouldering Rose",
        slug: "smoulderingRose",
        description: "Its as if magma is trying to seep through the stonelike flowers.",
        destroyOnHarvest: true,
        waterConsumptionRate: 4,
        growingTicks: 475,
        icon: FlowerImages.SmoulderingRose,
        loot: {
            conduitDropMin: 700,
            conduitDropMax: 900,
            possibleItemDrops: ["obsidianRoseSeed", "smoulderingRoseSeed"],
            guaranteedItemDrops: ["harvestedSmoulderingRose"],
            itemDropMin: 1,
            itemDropMax: 2,
        },
    },
    rainbowLotus: {
        name: "Rainbow Lotus",
        slug: "rainbowLotus",
        description: "A beautiful and delicate plant that thrives during rainbows.",
        destroyOnHarvest: true,
        waterConsumptionRate: 3,
        growingTicks: 72,
        icon: FlowerImages.RainbowLotus,
        loot: {
            conduitDropMin: 100,
            conduitDropMax: 250,
            possibleItemDrops: ["rainbowLotusSeed", "harvestedRainbowLotus"],
            guaranteedItemDrops: ["harvestedRainbowLotus"],
            itemDropMin: 0,
            itemDropMax: 2,
        },
    },
    dragonleaf: {
        name: "Dragonleaf",
        slug: "dragonleaf",
        description: "It produces smoke as it grows, drying itself out. It has to work much harder to stay alive during cold weather.",
        destroyOnHarvest: true,
        waterConsumptionRate: 10,
        growingTicks: 32,
        icon: FlowerImages.Dragonleaf,
        loot: {
            conduitDropMin: 250,
            conduitDropMax: 450,
            possibleItemDrops: [
                "dragonleafSeed",
                "goldLeaf",
                "leafPile",
                "fallenStar",
            ],
            guaranteedItemDrops: ["harvestedDragonleaf"],
            itemDropMin: 1,
            itemDropMax: 3,
        },
    },
    wanderingIce: {
        name: "Wandering Ice",
        slug: "wanderingIce",
        description: "Its as if an icicle has grown from the ground, but it sprouts leaves and vines just like any other plant. But its made of water and it has to stay cold, so, tend to it well! Loses twice as much water during hours with hot weather.",
        destroyOnHarvest: true,
        waterConsumptionRate: 9,
        growingTicks: 48,
        icon: FlowerImages.WanderingIce,
        loot: {
            conduitDropMin: 50,
            conduitDropMax: 100,
            possibleItemDrops: [
                "livingIcicle",
                "livingIcicle",
                "livingIcicle",
                "livingIcicle",
                "livingIcicle",
                "wanderingIceSeed",
            ],
            guaranteedItemDrops: ["livingIcicle"],
            itemDropMin: 2,
            itemDropMax: 4,
        },
    },
    lightningLotus: {
        name: "Lightning Lotus",
        slug: "lightningLotus",
        description: "Its wild energy is confounding, having spawned from pure lightning. Not... entirely sure... how watering it works but, yknow, see what happens I guess.",
        destroyOnHarvest: true,
        waterConsumptionRate: 6,
        growingTicks: 96,
        icon: FlowerImages.LightningLotus,
        loot: {
            conduitDropMin: 20,
            conduitDropMax: 100,
            possibleItemDrops: ["lightningInABottle"],
            guaranteedItemDrops: ["harvestedLightningLotus"],
            itemDropMin: 1,
            itemDropMax: 3,
        },
    },
    stardustBlossom: {
        name: "Stardust Blossom",
        slug: "stardustBlossom",
        description: "It's flowers sparkle especially bright under starlight",
        destroyOnHarvest: true,
        waterConsumptionRate: 5,
        growingTicks: 18,
        icon: FlowerImages.StardustBlossom,
        loot: {
            conduitDropMin: 5,
            conduitDropMax: 8,
            possibleItemDrops: ["stardustBag"],
            guaranteedItemDrops: ["stardustBag"],
            itemDropMin: 1,
            itemDropMax: 3,
        },
    },
    celestialBulb: {
        name: "Celestial Bulb",
        slug: "celestialBulb",
        description: "A large bulb growing from material found beyond the stars. I wonder what's inside?",
        destroyOnHarvest: true,
        waterConsumptionRate: 18,
        growingTicks: 6,
        icon: FlowerImages.CelestialBulb,
        loot: {
            conduitDropMin: 10,
            conduitDropMax: 20,
            possibleItemDrops: [
                "flaskSmall",
                "swirlingStormwrangler",
                "harvestedSmoulderingRose",
                "stardustBlossomSeed",
                "discardedJacket",
                "randomScrap",
            ],
            guaranteedItemDrops: ["stardustBag"],
            itemDropMin: 2,
            itemDropMax: 3,
        },
    },
    celestialBoonflower: {
        name: "Celestial Boonflower",
        slug: "celestialBoonflower",
        description: "An enormous flower that grows from material found beyond the stars. I wonder what's inside?",
        destroyOnHarvest: true,
        waterConsumptionRate: 45,
        growingTicks: 3,
        icon: FlowerImages.CelestialBoonflower,
        loot: {
            conduitDropMin: 20,
            conduitDropMax: 50,
            possibleItemDrops: [
                "flaskSmall",
                "swirlingStormwrangler",
                "harvestedSmoulderingRose",
                "stardustBlossomSeed",
                "discardedJacket",
                "randomScrap",
                "randomScrap",
                "livingIcicle",
                "lightningInABottle",
                "harvestedDragonleaf",
            ],
            guaranteedItemDrops: ["celestialBulbSeed", "celestialBulbSeed"],
            itemDropMin: 2,
            itemDropMax: 4,
        },
    },
    ritualPumpkin: {
        name: "Ritual Pumpkin",
        slug: "ritualPumpkin",
        description: "Who carved that face into it? What? Why is it already glowing?",
        destroyOnHarvest: true,
        waterConsumptionRate: 25,
        growingTicks: 9,
        icon: FlowerImages.RitualPumpkin,
        loot: {
            conduitDropMin: 100,
            conduitDropMax: 250,
            possibleItemDrops: ["harvestedRitualPumpkin"],
            guaranteedItemDrops: ["harvestedRitualPumpkin", "ritualPumpkinSeed"],
            itemDropMin: 0,
            itemDropMax: 0,
        },
    },
    astralMangoTree: {
        name: "Astral Mango Tree",
        slug: "astralMangoTree",
        description: "A tree with swirling bark that fruits with Astral Mangoes. After the first harvest, fruit is produced every 48 World Ticks.",
        destroyOnHarvest: false,
        regrowthTicks: 66,
        waterConsumptionRate: 4,
        growingTicks: 120,
        icon: FlowerImages.AstralMangoTree,
        loot: {
            conduitDropMin: 20,
            conduitDropMax: 50,
            possibleItemDrops: ["astralMango"],
            guaranteedItemDrops: ["astralMango"],
            itemDropMin: 1,
            itemDropMax: 2,
        },
    },
    spectralPapayaTree: {
        name: "Spectral Papaya Tree",
        slug: "spectralPapayaTree",
        description: "I think this tree might be haunting you. After the first harvest, fruit is produced every 70 World Ticks.",
        destroyOnHarvest: false,
        regrowthTicks: 70,
        waterConsumptionRate: 3,
        growingTicks: 160,
        icon: FlowerImages.SpectralPapayaTree,
        loot: {
            conduitDropMin: 10,
            conduitDropMax: 80,
            possibleItemDrops: ["spectralPapaya"],
            guaranteedItemDrops: ["spectralPapaya"],
            itemDropMin: 1,
            itemDropMax: 3,
        },
    },
    acidBlossom: {
        name: "Acid Blossom",
        slug: "acidBlossom",
        description: "The leaves bubble and gurgle with acid, emittng a pungent yet slightly sweet smell. Acid Blossoms only grow during Acid Rain and will nullify the effects of Acid Rain on your other crops.",
        destroyOnHarvest: true,
        waterConsumptionRate: 0,
        growingTicks: 2,
        icon: FlowerImages.AcidBlossom,
        loot: {
            conduitDropMin: 300,
            conduitDropMax: 800,
            possibleItemDrops: ["cracklingRainmeter"],
            guaranteedItemDrops: ["harvestedAcidBlossom"],
            itemDropMin: 1,
            itemDropMax: 2,
        },
    },
    trophyThistle: {
        name: "Trophy Thistle",
        slug: "trophyThistle",
        description: "Among the most beautiful of thistles, this one is a real showstopper.",
        destroyOnHarvest: true,
        waterConsumptionRate: 11,
        growingTicks: 744,
        icon: FlowerImages.TrophyThistle,
        loot: {
            conduitDropMin: 5000,
            conduitDropMax: 10000,
            possibleItemDrops: ["trophyThistleSeed"],
            guaranteedItemDrops: ["harvestedTrophyThistle", "trophyThistleSeed"],
            itemDropMin: 0,
            itemDropMax: 0,
        },
    },
    joyroot: {
        name: "Joyroot",
        slug: "joyroot",
        description: "This plant has petrified as it is no longer the Stellarvibe season.",
        destroyOnHarvest: true,
        waterConsumptionRate: 1,
        growingTicks: 2,
        icon: FlowerImages.Joyroot,
        loot: {
            conduitDropMin: 100,
            conduitDropMax: 200,
            possibleItemDrops: [],
            guaranteedItemDrops: [],
            itemDropMin: 0,
            itemDropMax: 0,
        },
    },
    joyblossom: {
        name: "Joyblossom",
        slug: "joyblossom",
        description: "This plant has petrified as it is no longer the Stellarvibe season.",
        waterConsumptionRate: 1,
        destroyOnHarvest: true,
        growingTicks: 2,
        icon: FlowerImages.Joyblossom,
        loot: {
            conduitDropMin: 100,
            conduitDropMax: 200,
            possibleItemDrops: [],
            guaranteedItemDrops: [],
            itemDropMin: 0,
            itemDropMax: 0,
        },
    },
    joybrushTangle: {
        name: "Joybrush Tangle",
        slug: "joybrushTangle",
        description: "This plant has petrified as it is no longer the Stellarvibe season.",
        waterConsumptionRate: 1,
        destroyOnHarvest: true,
        growingTicks: 2,
        icon: FlowerImages.JoybrushTangle,
        loot: {
            conduitDropMin: 100,
            conduitDropMax: 200,
            possibleItemDrops: [],
            guaranteedItemDrops: [],
            itemDropMin: 0,
            itemDropMax: 0,
        },
    },
};
export { FLOWER_DEFINITIONS };
export function getFlowerDefinition(slug) {
    return FLOWER_DEFINITIONS[slug];
}
