import { z } from "zod";
import { CamperChipSchema } from "./core";
/*******************
 * MASTODON SHAPES *
 *******************/
/**
 * An identity object for a Mastodon account associated with a camper
 */
export const MastodonIdentitySchema = z.object({
    mastodonIdentityId: z.number(),
    mastodonId: z.string(),
    camper: CamperChipSchema.nullable(),
    token: z.string().nullable(),
    handle: z.string(),
});
/**
 * An attachment object for a Mastodon status
 */
export const MastodonStatusAttachmentSchema = z.object({
    type: z.enum(["image", "video", "gifv", "audio", "unknown"]),
    url: z.string(),
    description: z.string().nullable(),
});
/**
 * A Mastodon status as represented in the Relay
 */
export const MastodonStatusSchema = z.object({
    id: z.number(),
    mastodonId: z.string(),
    createdAt: z.string(),
    authorIdentity: MastodonIdentitySchema,
    inReplyToId: z.string().nullable(),
    inReplyToAccountId: z.string().nullable(),
    contentWarning: z.string().nullable(),
    visibility: z.string(),
    language: z.string().nullable(),
    url: z.string(),
    replyCount: z.number(),
    repostCount: z.number(),
    favoriteCount: z.number(),
    editedAt: z.string().nullable(),
    content: z.string(),
    repostedStatusId: z.string().nullable(),
    attachments: z.array(MastodonStatusAttachmentSchema),
});
/******************************
 * MASTODN API RESPONSE TYPES *
 ******************************/
/**
 * Represents the expected shape of a Mastodon API account object
 */
export const MastodonAPIAccountSchema = z.object({
    id: z.string(),
    username: z.string(),
});
/**
 * Represents the expected shape of a Mastodon tag in a status
 */
export const MastodonAPIStatusTagSchema = z.object({
    name: z.string(),
    url: z.string(),
});
/**
 * Represents the expected shape of a Mastodon media object in a status
 */
export const MastodonAPIStatusMediaSchema = z.object({
    id: z.string(),
    type: z.enum(["image", "video", "gifv", "audio", "unknown"]),
    url: z.string(),
    preview_url: z.string().nullable(),
    remote_url: z.string().nullable(),
    description: z.string().nullable(),
    blurhash: z.string().nullable(),
});
/**
 * A Mastodon API status object.
 * Convert to an internal MastodonStatus upon receiving
 */
export const MastodonAPIStatusSchema = z.object({
    account: MastodonAPIAccountSchema,
    id: z.string(),
    created_at: z.string(),
    in_reply_to_id: z.string().nullable(),
    in_reply_to_account_id: z.string().nullable(),
    sensitive: z.boolean(),
    spoiler_text: z.string(),
    visibility: z.string(),
    language: z.string().nullable(),
    uri: z.string(),
    url: z.string(),
    replies_count: z.number(),
    reblogs_count: z.number(),
    favourites_count: z.number(),
    edited_at: z.string().nullable(),
    content: z.string(),
    tags: z.array(MastodonAPIStatusTagSchema),
    reblog: z.any().nullable(),
    media_attachments: z.array(MastodonAPIStatusMediaSchema),
});
export const MastodonAPIStatusArraySchema = z.array(MastodonAPIStatusSchema);
/*******************
 * MASTODON EVENTS *
 *******************/
export const MastodonRelayEvents = {
    "mastodon:status": MastodonStatusSchema,
};
/********************
 * MASTODON ACTIONS *
 ********************/
export const MastodonRelayActions = {
    "mastodon@attach-identity": {
        input: z.object({
            token: z.string(),
        }),
        output: z.string(),
    },
    "mastodon@detach-identity": {
        input: z.void(),
        output: z.boolean(),
    },
    "mastodon@status": {
        input: z.object({
            status: z.string(),
        }),
        output: z.string(),
    },
};
