import { z } from "zod";
import { ColorStringSchema, PronounStringSchema, URLStringSchema, } from "./branded-primitives";
import { CamperChipSchema } from "./core";
/********************
 * TWITCH CONSTANTS *
 ********************/
/**
 * Flairs that can be applied to a chat message
 */
export const TWITCH_CHAT_FLAIRS = [
    "broadcaster",
    "vip",
    "mod",
    "highlighted",
    "subscriber",
    "camper",
    "bot",
    "firstThisSession",
    "firstEver",
];
/**
 * Scopes required for the Twitch API
 */
export const TWITCH_AUTH_SCOPE = [
    // Chat
    "chat:read",
    "chat:edit",
    // Bits
    "bits:read",
    // Moderator
    "moderator:read:followers",
    "moderator:manage:chat_messages",
    "moderator:manage:shoutouts",
    "moderator:manage:banned_users",
    // Channel
    "channel:read:hype_train",
    "channel:read:vips",
    "channel:read:redemptions",
    "channel:read:subscriptions",
    "channel:manage:raids",
    "channel:manage:redemptions",
    "channel:manage:vips",
    "channel:moderate",
    "channel:bot",
    // User
    "user:read:email",
    "user:bot",
].join("+");
export const TWITCH_AUTH_SCOPE_BASIC = ["user:read:email"].join("+");
/*****************
 * TWITCH SHAPES *
 *****************/
export const TwitchSubscriptionTierSchema = z.union([
    z.literal("1000"),
    z.literal("2000"),
    z.literal("3000"),
]);
/**
 * #SHAPE# TwitchIdentity
 *
 * An identity object for a Twitch account associated with a camper
 */
export const TwitchIdentitySchema = z
    .object({
    twitchIdentityId: z.number(),
    twitchId: z.string(),
    camper: CamperChipSchema.nullable(),
    twitchHandle: z.string(),
    twitchDisplayName: z.string(),
    twitchNameColor: z.string(),
    twitchChannelURL: z.string().url(),
    info: z.object({
        name: z.string(),
        color: ColorStringSchema,
        pronouns: PronounStringSchema.nullable(),
        avatarURL: URLStringSchema.nullable(),
    }),
})
    .readonly();
export const TwitchChatFlairSchema = z.enum(TWITCH_CHAT_FLAIRS);
export const TwitchCharFlairArraySchema = z.array(TwitchChatFlairSchema);
/**
 * A chat message in a Twitch channel as represented in the Relay
 */
export const TwitchChatSchema = z
    .object({
    chatRecordId: z.number(),
    twitchMessageId: z.string(),
    twitchParentMessageId: z.string().nullable(),
    authorName: z.string(),
    content: z.string(),
    richContent: z.string().nullable(),
    flair: TwitchCharFlairArraySchema,
    identity: TwitchIdentitySchema,
    camper: CamperChipSchema.nullable(),
})
    .readonly();
/****************
 * TWITCH VIEWS *
 ****************/
export const TwitchRelayViews = {
    "twitch/chat/history": {
        params: z.object({}),
        response: z.array(TwitchChatSchema),
    },
    "twitch/identity/by-id": {
        params: z.object({ id: z.number() }),
        response: z.nullable(TwitchIdentitySchema),
    },
    "twitch/identity/by-twitch-id": {
        params: z.object({ twitchId: z.string() }),
        response: z.nullable(TwitchIdentitySchema),
    },
    "twitch/identity/by-twitch-handle": {
        params: z.object({ twitchHandle: z.string() }),
        response: z.nullable(TwitchIdentitySchema),
    },
    "twitch/identity/by-camper-id": {
        params: z.object({ camperId: z.number() }),
        response: z.nullable(TwitchIdentitySchema),
    },
    // TODO: Are these actually needed here?
    "twitch/identities-with-campers": {
        params: z.object({}),
        response: z.array(TwitchIdentitySchema),
    },
    "twitch/identities-with-camper-and-token": {
        params: z.object({}),
        response: z.array(TwitchIdentitySchema),
    },
};
// -- EVENTS -- //
export const TwitchRelayEvents = {
    "twitch:chat": z.object({
        chat: TwitchChatSchema,
        channel: TwitchIdentitySchema,
    }),
    "twitch:follow": z.object({
        channel: TwitchIdentitySchema,
        follower: TwitchIdentitySchema,
    }),
    "twitch:live": z.object({
        channel: TwitchIdentitySchema,
    }),
    "twitch:raid": z.object({
        channel: TwitchIdentitySchema,
        raider: TwitchIdentitySchema,
        count: z.number(),
    }),
    "twitch:hype-train-start": z.object({
        channel: TwitchIdentitySchema,
    }),
    "twitch:hype-train-end": z.object({
        channel: TwitchIdentitySchema,
        level: z.number(),
    }),
    "twitch:cheer": z.object({
        channel: TwitchIdentitySchema,
        cheerer: TwitchIdentitySchema,
        bits: z.number(),
        message: z.string().nullable(),
    }),
    "twitch:ban": z.object({
        channel: TwitchIdentitySchema,
        target: TwitchIdentitySchema,
        reason: z.string(),
    }),
    "twitch:subscription": z.object({
        channel: TwitchIdentitySchema,
        subscriber: TwitchIdentitySchema,
        months: z.number(),
        message: z.string().nullable(),
        isGift: z.boolean(),
        tier: TwitchSubscriptionTierSchema,
    }),
    "twitch:gifted-subscriptions": z.object({
        channel: TwitchIdentitySchema,
        gifter: TwitchIdentitySchema.nullable(),
        count: z.number(),
        tier: TwitchSubscriptionTierSchema,
    }),
    "twitch:redemption": z.object({
        channel: TwitchIdentitySchema,
        viewer: TwitchIdentitySchema,
        redeemable: z.object({
            id: z.string(),
            title: z.string(),
            message: z.string().optional(),
        }),
    }),
};
/******************
 * TWITCH ACTIONS *
 ******************/
export const TwitchRelayActions = {
    "twitch@chat": {
        input: z.object({
            channelTwitchId: z.string(),
            content: z.string(),
        }),
        output: z.boolean(),
    },
    "twitch@delete-chat-record": {
        input: z.object({
            messageTwitchId: z.string(),
        }),
        output: z.boolean(),
    },
    "twitch@shoutout": {
        input: z.string(),
        output: z.boolean(),
    },
    "twitch@clear-chat": {
        input: z.undefined(),
        output: z.boolean(),
    },
};
